requirejs.config({
    paths: {
        jquery: 'CodeBlockProgrammingScore/runtime/js/lib/jquery.min'
    }
});
define([
    'jquery',
    'qtiCustomInteractionContext',
    'OAT/util/html',
    'OAT/util/event',
], function ($, qtiCustomInteractionContext, html, event) {

    let PciManager = function () {
        this.typeIdentifier = null;
        this.response = {base: {string: ''}};
    };

    PciManager.prototype = {

        getBase: function () {
            let self = this;

            return {

                /**
                 * Custom Interaction Hook API: id
                 */
                id: -1,

                /**
                 * Custom Interaction Hook API: getTypeIdentifier
                 *
                 * A unique identifier allowing custom interactions to be identified within an item.
                 *
                 * @returns {String} The unique identifier of this PCI Hook implementation.
                 */
                getTypeIdentifier: function () {
                    return self.typeIdentifier;
                },


                /**
                 *
                 *
                 * @param {String} id
                 * @param {Node} dom
                 * @param {Object} config - json
                 */
                initialize: function (id, dom, config, assetManager) {

                    //add method on(), off() and trigger() to the current object
                    event.addEventMgr(this);

                    // Register the value for the 'id' attribute of this Custom Interaction Hook instance.
                    // We consider in this proposal that the 'id' attribute
                    this.id = id;
                    this.dom = dom;
                    this.config = config || {};
                    this.$container = $(dom);
                },

                notifyReady: function () {
                    qtiCustomInteractionContext.notifyReady(this);
                },

                /**
                 * Programmatically set the response following the json schema described in
                 * http://www.imsglobal.org/assessment/pciv1p0cf/imsPCIv1p0cf.html#_Toc353965343
                 *
                 * @param {Object} interaction
                 * @param {Object} response
                 */
                setResponse: function (response) {
                    self.response = response;
                },

                /**
                 * Get the response in the json format described in
                 * http://www.imsglobal.org/assessment/pciv1p0cf/imsPCIv1p0cf.html#_Toc353965343
                 *
                 * @param {Object} interaction
                 * @returns {Object}
                 */
                getResponse: function () {
                    return self.response;
                },

                /**
                 * Remove the current response set in the interaction
                 * The state may not be restored at this point.
                 *
                 * @param {Object} interaction
                 */
                resetResponse: function () {
                    self.response = {base: {string: ''}};
                },

                /**
                 * Reverse operation performed by render()
                 * After this function is executed, only the inital naked markup remains
                 * Event listeners are removed and the state and the response are reset
                 *
                 * @param {Object} interaction
                 */
                destroy: function () {
                    let $container = $(this.dom);
                    $container.off().empty();
                },

                /**
                 * Restore the state of the interaction from the serializedState.
                 *
                 * @param {Object} interaction
                 * @param {Object} serializedState - json format
                 */
                setSerializedState: function (state) {
                    if (state && state.response) {
                        self.response = state.response;
                    }
                },

                /**
                 * Get the current state of the interaction as a string.
                 * It enables saving the state for later usage.
                 *
                 * @param {Object} interaction
                 * @returns {Object} json format
                 */
                getSerializedState: function () {
                    return {response: this.getResponse()};
                },

            };
        },

        updateResponse : function(responseVal){
            this.response = {
                base: {
                    string: responseVal
                }
            };
        },

        register: function (pciObject) {
            return qtiCustomInteractionContext.register(pciObject);
        },

        assignObject: function (base) {
            if (typeof Object.assign != 'function') {
                Object.assign = function (target) {
                    'use strict';
                    if (target == null) {
                        throw new TypeError('Cannot convert undefined or null to object');
                    }

                    target = Object(target);
                    for (let index = 1; index < arguments.length; index++) {
                        let source = arguments[index];
                        if (source != null) {
                            for (let key in source) {
                                if (Object.prototype.hasOwnProperty.call(source, key)) {
                                    target[key] = source[key];
                                }
                            }
                        }
                    }
                    return target;
                };
            }

            return Object.assign({}, base);
        },
    };

    return PciManager;
});
